/*

	SDL video backend for Windows, *nix and Dingux
	For handheld consoles better use m_gfx_dingoosdl.c
  
*/

#include "m_core.h"
#include "m_gfx.h"
#include "SDL/SDL.h"

// forward def of palette in RGBA format
extern unsigned char pal256[1024];

unsigned char Keys[128] = {0};
unsigned char ScreenBuffer[320*200] = {0};

#if defined(__DINGUX__) || defined(__GP2X__) || defined(__CAANOO__)
int scale2x = 1;
int fullscr = 0;
#elif defined(__WIN32__) || defined(__UNIX__)
int scale2x = 2; // 1 - no scale 320x200; 2 - upscale to 640x400
int fullscr = 0; // or SDL_FULLSCREEN
int _toggle = 1;
#else
	#error Platform not defined or supported
#endif

int LM_GFX_Init();
void LM_GFX_Deinit();

SDL_Surface *screen = NULL;

#if defined(__DINGUX__) || defined(__GP2X__) || defined(__CAANOO__)
SDL_Surface *hardwareScreen = NULL;
#endif

//===============================================================
void LM_ResetKeys()
{
	memset(&Keys[0], 0, 128);
}

int LM_AnyKey()
{
    for(int i = 0; i < 127; i++)
	{
        if(Keys[i] == 1) return 1;
    }
    return 0;
}

int LM_Timer()
{
	return SDL_GetTicks();
}

void LM_Sleep(int sleep_time)
{
	SDL_Delay(sleep_time); 
}

int LM_Init(unsigned char **pScreenBuffer)
{
	if(LM_GFX_Init() == 0) return 0;
	*pScreenBuffer = &ScreenBuffer[0];
	return 1;
}

void LM_Deinit()
{
	LM_GFX_Deinit();
}

char LM_PollEvents()
{
	SDL_Event event;
   
	while (SDL_PollEvent(&event) != 0)
	{ 
		#ifndef __WIN32__
		int key_scan = -1;
		unsigned char key_value = 0;
		#endif

		if(event.type == SDL_QUIT) {Keys[SC_ESCAPE] = 1; return 1; }
		
		// only valid for i386 arch and Windows
		#ifdef __WIN32__
		if(event.type == SDL_KEYDOWN )
		{
			Keys[event.key.keysym.scancode] = 1;
		} else if(event.type == SDL_KEYUP)
		{
			Keys[event.key.keysym.scancode] = 0;
		}
		#endif

		// unix and dingoo sdl don't have scancodes, so remap usual keys
		#if defined(__UNIX__) || defined(__DINGUX__) || defined(__GP2X__) || defined(__CAANOO__)
		if(event.type == SDL_KEYDOWN) key_value = 1;
		if(event.type == SDL_KEYUP) key_value = 0;
		
		// Emulate x86 scancodes
		if(event.type == SDL_KEYDOWN || event.type == SDL_KEYUP)
		{					
			switch(event.key.keysym.sym)
			{
				case SDLK_UP:
					key_scan = SC_UP;	
					break;
				case SDLK_DOWN:
					key_scan = SC_DOWN;
					break;
				case SDLK_LEFT:
					key_scan = SC_LEFT;
					break;				
				case SDLK_RIGHT:
					key_scan = SC_RIGHT;
					break;					
				case SDLK_ESCAPE:
					key_scan = SC_ESCAPE;
					break;
				case SDLK_RETURN: // ENTER
					key_scan = SC_ENTER;
					break;					
				case SDLK_LCTRL:
				case SDLK_LALT:
				case SDLK_LSHIFT:				
				case SDLK_SPACE:
					key_scan = SC_SPACE;
					break;	
				case SDLK_s:
					key_scan = SC_S;
					break;
				case SDLK_f:
					key_scan = SC_F;
					break;				
				case SDLK_TAB: // LEFT SHOULDER
					key_scan = SC_TAB;
					break;
				case SDLK_BACKSPACE: // RIGHT SHOULDER
					key_scan = SC_BACKSPACE;
					break;					
				default:; // maybe use in future
					break;
			}			
			if(key_scan != -1) Keys[key_scan] = key_value;
		}
		#endif
    } 
	
	// toggle sizes x1 or x2 with scanlines
	// for win32 and unix only
	#if defined(__WIN32__) || defined(__UNIX__)
	if(Keys[SC_F] == 1) {
		fullscr ^= SDL_FULLSCREEN; 
		Keys[SC_F] = 0; 
		LM_GFX_SetScale(2);
		return 0;
	}
	if(Keys[SC_S] == 1 && fullscr == 0) {Keys[SC_S] = 0; _toggle ^= 1; LM_GFX_SetScale(_toggle + 1);}	
	#endif

	return 0;
}

int LM_GFX_Init()
{
	//Start SDL 
	//if(SDL_Init(SDL_INIT_EVERYTHING) < 0) return 0; 
	if(SDL_Init(SDL_INIT_VIDEO | SDL_INIT_TIMER | SDL_INIT_AUDIO) < 0) return 0; 
	atexit(SDL_Quit); 
	
#if defined(__DINGUX__) || defined(__GP2X__) || defined(__CAANOO__)
	screen = SDL_CreateRGBSurface(SDL_SWSURFACE, 320, 240, 8, 0, 0, 0, 0);
	SDL_SetPalette(screen, SDL_LOGPAL, (SDL_Color *)pal256, 0, 256);		
	hardwareScreen = SDL_SetVideoMode(320, 240, 16, SDL_SWSURFACE | fullscr);	
	
	SDL_ShowCursor(SDL_DISABLE);
#elif defined(__WIN32__) || defined (__UNIX__)
	screen = SDL_SetVideoMode(640, 480, 8, SDL_SWSURFACE | fullscr);
	SDL_SetPalette(screen, SDL_PHYSPAL, (SDL_Color *)pal256, 0, 256);		
#else
	#error Platform not defined or supported
#endif
	
	//Set the window caption 
	SDL_WM_SetCaption("The Last Mission SDL remake", NULL); 
	
	return 1;
}

void LM_GFX_Deinit()
{
	//SDL_Quit();
}

void LM_GFX_Flip(unsigned char *p)
{
#if defined(__DINGUX__) || defined(__GP2X__) || defined(__CAANOO__)
	memcpy(screen->pixels + 320*20, p, 320*200);
	SDL_BlitSurface(screen, NULL, hardwareScreen, NULL);
	SDL_Flip(hardwareScreen);	
#elif defined(__WIN32__) || (__UNIX__)
	if(scale2x == 2)
	{
		for(int y = 0; y <= 199; y++)
		{
			for(int x = 0; x <= 319; x++)
			{
				unsigned char tmp = *(unsigned char *)(p + y*320+x);
				unsigned char *pDest = (unsigned char *)screen->pixels;

				*(pDest + 640*40 + (y*2+1)*640+x*2) = tmp;
				*(pDest + 640*40 + (y*2+1)*640+x*2+1) = tmp;
				*(pDest + 640*40 + y*2*640+x*2) = tmp;
				*(pDest + 640*40 + y*2*640+x*2+1) = tmp;
			}
		}

	}
	else
	{
		memcpy(screen->pixels + 320*20, p, 320*200);
	}

	//Update Screen 
	SDL_Flip(screen); 
#else
	#error Platform not defined or supported
#endif
}

void LM_GFX_WaitVSync()
{
}

void LM_GFX_SetScale(int param)
{
#if defined(__WIN32__) || (__UNIX__)
    scale2x = ((param - 1) & 1) + 1; // ensure param to be 1 or 2 strictly
	if(fullscr == SDL_FULLSCREEN) scale2x = 2;
	
	screen = SDL_SetVideoMode(320 * scale2x, 240 * scale2x, 8, SDL_SWSURFACE | fullscr);	
	SDL_SetPalette(screen, SDL_PHYSPAL, (SDL_Color *)pal256, 0, 256);
	LM_ResetKeys();
#endif
}

unsigned char pal256[1024] = {
	 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xAA, 0x00, 0x00, 0xAA, 0x00, 0x00, 0x00, 0xAA, 0xAA, 0x00, 
	 0xAA, 0x00, 0x00, 0x00, 0xAA, 0x00, 0xAA, 0x00, 0xAA, 0x55, 0x00, 0x00, 0xAA, 0xAA, 0xAA, 0x00, 
	 0x55, 0x55, 0x55, 0x00, 0x55, 0x55, 0xFF, 0x00, 0x55, 0xFF, 0x55, 0x00, 0x55, 0xFF, 0xFF, 0x00, 
	 0xFF, 0x55, 0x55, 0x00, 0xFF, 0x55, 0xFF, 0x00, 0xFF, 0xFF, 0x55, 0x00, 0xFF, 0xFF, 0xFF, 0x00, 
	 0x00, 0x00, 0x00, 0x00, 0x14, 0x14, 0x14, 0x00, 0x20, 0x20, 0x20, 0x00, 0x2C, 0x2C, 0x2C, 0x00, 
	 0x38, 0x38, 0x38, 0x00, 0x44, 0x44, 0x44, 0x00, 0x50, 0x50, 0x50, 0x00, 0x61, 0x61, 0x61, 0x00, 
	 0x71, 0x71, 0x71, 0x00, 0x81, 0x81, 0x81, 0x00, 0x91, 0x91, 0x91, 0x00, 0xA1, 0xA1, 0xA1, 0x00, 
	 0xB6, 0xB6, 0xB6, 0x00, 0xCA, 0xCA, 0xCA, 0x00, 0xE2, 0xE2, 0xE2, 0x00, 0xFF, 0xFF, 0xFF, 0x00, 
	 0x00, 0x00, 0xFF, 0x00, 0x40, 0x00, 0xFF, 0x00, 0x7D, 0x00, 0xFF, 0x00, 0xBE, 0x00, 0xFF, 0x00, 
	 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xBE, 0x00, 0xFF, 0x00, 0x7D, 0x00, 0xFF, 0x00, 0x40, 0x00, 
	 0xFF, 0x00, 0x00, 0x00, 0xFF, 0x40, 0x00, 0x00, 0xFF, 0x7D, 0x00, 0x00, 0xFF, 0xBE, 0x00, 0x00, 
	 0xFF, 0xFF, 0x00, 0x00, 0xBE, 0xFF, 0x00, 0x00, 0x7D, 0xFF, 0x00, 0x00, 0x40, 0xFF, 0x00, 0x00, 
	 0x00, 0xFF, 0x00, 0x00, 0x00, 0xFF, 0x40, 0x00, 0x00, 0xFF, 0x7D, 0x00, 0x00, 0xFF, 0xBE, 0x00, 
	 0x00, 0xFF, 0xFF, 0x00, 0x00, 0xBE, 0xFF, 0x00, 0x00, 0x7D, 0xFF, 0x00, 0x00, 0x40, 0xFF, 0x00, 
	 0x7D, 0x7D, 0xFF, 0x00, 0x9D, 0x7D, 0xFF, 0x00, 0xBE, 0x7D, 0xFF, 0x00, 0xDE, 0x7D, 0xFF, 0x00, 
	 0xFF, 0x7D, 0xFF, 0x00, 0xFF, 0x7D, 0xDE, 0x00, 0xFF, 0x7D, 0xBE, 0x00, 0xFF, 0x7D, 0x9D, 0x00, 
	 0xFF, 0x7D, 0x7D, 0x00, 0xFF, 0x9D, 0x7D, 0x00, 0xFF, 0xBE, 0x7D, 0x00, 0xFF, 0xDE, 0x7D, 0x00, 
	 0xFF, 0xFF, 0x7D, 0x00, 0xDE, 0xFF, 0x7D, 0x00, 0xBE, 0xFF, 0x7D, 0x00, 0x9D, 0xFF, 0x7D, 0x00, 
	 0x7D, 0xFF, 0x7D, 0x00, 0x7D, 0xFF, 0x9D, 0x00, 0x7D, 0xFF, 0xBE, 0x00, 0x7D, 0xFF, 0xDE, 0x00, 
	 0x7D, 0xFF, 0xFF, 0x00, 0x7D, 0xDE, 0xFF, 0x00, 0x7D, 0xBE, 0xFF, 0x00, 0x7D, 0x9D, 0xFF, 0x00, 
	 0xB6, 0xB6, 0xFF, 0x00, 0xC6, 0xB6, 0xFF, 0x00, 0xDA, 0xB6, 0xFF, 0x00, 0xEA, 0xB6, 0xFF, 0x00, 
	 0xFF, 0xB6, 0xFF, 0x00, 0xFF, 0xB6, 0xEA, 0x00, 0xFF, 0xB6, 0xDA, 0x00, 0xFF, 0xB6, 0xC6, 0x00, 
	 0xFF, 0xB6, 0xB6, 0x00, 0xFF, 0xC6, 0xB6, 0x00, 0xFF, 0xDA, 0xB6, 0x00, 0xFF, 0xEA, 0xB6, 0x00, 
	 0xFF, 0xFF, 0xB6, 0x00, 0xEA, 0xFF, 0xB6, 0x00, 0xDA, 0xFF, 0xB6, 0x00, 0xC6, 0xFF, 0xB6, 0x00, 
	 0xB6, 0xFF, 0xB6, 0x00, 0xB6, 0xFF, 0xC6, 0x00, 0xB6, 0xFF, 0xDA, 0x00, 0xB6, 0xFF, 0xEA, 0x00, 
	 0xB6, 0xFF, 0xFF, 0x00, 0xB6, 0xEA, 0xFF, 0x00, 0xB6, 0xDA, 0xFF, 0x00, 0xB6, 0xC6, 0xFF, 0x00, 
	 0x00, 0x00, 0x71, 0x00, 0x1C, 0x00, 0x71, 0x00, 0x38, 0x00, 0x71, 0x00, 0x55, 0x00, 0x71, 0x00, 
	 0x71, 0x00, 0x71, 0x00, 0x71, 0x00, 0x55, 0x00, 0x71, 0x00, 0x38, 0x00, 0x71, 0x00, 0x1C, 0x00, 
	 0x71, 0x00, 0x00, 0x00, 0x71, 0x1C, 0x00, 0x00, 0x71, 0x38, 0x00, 0x00, 0x71, 0x55, 0x00, 0x00, 
	 0x71, 0x71, 0x00, 0x00, 0x55, 0x71, 0x00, 0x00, 0x38, 0x71, 0x00, 0x00, 0x1C, 0x71, 0x00, 0x00, 
	 0x00, 0x71, 0x00, 0x00, 0x00, 0x71, 0x1C, 0x00, 0x00, 0x71, 0x38, 0x00, 0x00, 0x71, 0x55, 0x00, 
	 0x00, 0x71, 0x71, 0x00, 0x00, 0x55, 0x71, 0x00, 0x00, 0x38, 0x71, 0x00, 0x00, 0x1C, 0x71, 0x00, 
	 0x38, 0x38, 0x71, 0x00, 0x44, 0x38, 0x71, 0x00, 0x55, 0x38, 0x71, 0x00, 0x61, 0x38, 0x71, 0x00, 
	 0x71, 0x38, 0x71, 0x00, 0x71, 0x38, 0x61, 0x00, 0x71, 0x38, 0x55, 0x00, 0x71, 0x38, 0x44, 0x00, 
	 0x71, 0x38, 0x38, 0x00, 0x71, 0x44, 0x38, 0x00, 0x71, 0x55, 0x38, 0x00, 0x71, 0x61, 0x38, 0x00, 
	 0x71, 0x71, 0x38, 0x00, 0x61, 0x71, 0x38, 0x00, 0x55, 0x71, 0x38, 0x00, 0x44, 0x71, 0x38, 0x00, 
	 0x38, 0x71, 0x38, 0x00, 0x38, 0x71, 0x44, 0x00, 0x38, 0x71, 0x55, 0x00, 0x38, 0x71, 0x61, 0x00, 
	 0x38, 0x71, 0x71, 0x00, 0x38, 0x61, 0x71, 0x00, 0x38, 0x55, 0x71, 0x00, 0x38, 0x44, 0x71, 0x00, 
	 0x50, 0x50, 0x71, 0x00, 0x59, 0x50, 0x71, 0x00, 0x61, 0x50, 0x71, 0x00, 0x69, 0x50, 0x71, 0x00, 
	 0x71, 0x50, 0x71, 0x00, 0x71, 0x50, 0x69, 0x00, 0x71, 0x50, 0x61, 0x00, 0x71, 0x50, 0x59, 0x00, 
	 0x71, 0x50, 0x50, 0x00, 0x71, 0x59, 0x50, 0x00, 0x71, 0x61, 0x50, 0x00, 0x71, 0x69, 0x50, 0x00,  
	 0x71, 0x71, 0x50, 0x00, 0x69, 0x71, 0x50, 0x00, 0x61, 0x71, 0x50, 0x00, 0x59, 0x71, 0x50, 0x00, 
	 0x50, 0x71, 0x50, 0x00, 0x50, 0x71, 0x59, 0x00, 0x50, 0x71, 0x61, 0x00, 0x50, 0x71, 0x69, 0x00, 
	 0x50, 0x71, 0x71, 0x00, 0x50, 0x69, 0x71, 0x00, 0x50, 0x61, 0x71, 0x00, 0x50, 0x59, 0x71, 0x00, 
	 0x00, 0x00, 0x40, 0x00, 0x10, 0x00, 0x40, 0x00, 0x20, 0x00, 0x40, 0x00, 0x30, 0x00, 0x40, 0x00, 
	 0x40, 0x00, 0x40, 0x00, 0x40, 0x00, 0x30, 0x00, 0x40, 0x00, 0x20, 0x00, 0x40, 0x00, 0x10, 0x00, 
	 0x40, 0x00, 0x00, 0x00, 0x40, 0x10, 0x00, 0x00, 0x40, 0x20, 0x00, 0x00, 0x40, 0x30, 0x00, 0x00, 
	 0x40, 0x40, 0x00, 0x00, 0x30, 0x40, 0x00, 0x00, 0x20, 0x40, 0x00, 0x00, 0x10, 0x40, 0x00, 0x00, 
	 0x00, 0x40, 0x00, 0x00, 0x00, 0x40, 0x10, 0x00, 0x00, 0x40, 0x20, 0x00, 0x00, 0x40, 0x30, 0x00, 
	 0x00, 0x40, 0x40, 0x00, 0x00, 0x30, 0x40, 0x00, 0x00, 0x20, 0x40, 0x00, 0x00, 0x10, 0x40, 0x00, 
	 0x20, 0x20, 0x40, 0x00, 0x28, 0x20, 0x40, 0x00, 0x30, 0x20, 0x40, 0x00, 0x38, 0x20, 0x40, 0x00, 
	 0x40, 0x20, 0x40, 0x00, 0x40, 0x20, 0x38, 0x00, 0x40, 0x20, 0x30, 0x00, 0x40, 0x20, 0x28, 0x00, 
	 0x40, 0x20, 0x20, 0x00, 0x40, 0x28, 0x20, 0x00, 0x40, 0x30, 0x20, 0x00, 0x40, 0x38, 0x20, 0x00, 
	 0x40, 0x40, 0x20, 0x00, 0x38, 0x40, 0x20, 0x00, 0x30, 0x40, 0x20, 0x00, 0x28, 0x40, 0x20, 0x00, 
	 0x20, 0x40, 0x20, 0x00, 0x20, 0x40, 0x28, 0x00, 0x20, 0x40, 0x30, 0x00, 0x20, 0x40, 0x38, 0x00, 
	 0x20, 0x40, 0x40, 0x00, 0x20, 0x38, 0x40, 0x00, 0x20, 0x30, 0x40, 0x00, 0x20, 0x28, 0x40, 0x00, 
	 0x2C, 0x2C, 0x40, 0x00, 0x30, 0x2C, 0x40, 0x00, 0x34, 0x2C, 0x40, 0x00, 0x3C, 0x2C, 0x40, 0x00, 
	 0x40, 0x2C, 0x40, 0x00, 0x40, 0x2C, 0x3C, 0x00, 0x40, 0x2C, 0x34, 0x00, 0x40, 0x2C, 0x30, 0x00, 
	 0x40, 0x2C, 0x2C, 0x00, 0x40, 0x30, 0x2C, 0x00, 0x40, 0x34, 0x2C, 0x00, 0x40, 0x3C, 0x2C, 0x00, 
	 0x40, 0x40, 0x2C, 0x00, 0x3C, 0x40, 0x2C, 0x00, 0x34, 0x40, 0x2C, 0x00, 0x30, 0x40, 0x2C, 0x00, 
	 0x2C, 0x40, 0x2C, 0x00, 0x2C, 0x40, 0x30, 0x00, 0x2C, 0x40, 0x34, 0x00, 0x2C, 0x40, 0x3C, 0x00, 
	 0x2C, 0x40, 0x40, 0x00, 0x2C, 0x3C, 0x40, 0x00, 0x2C, 0x34, 0x40, 0x00, 0x2C, 0x30, 0x40, 0x00, 
	 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 
};